function getTextWidth(text, fontSize, fontName) {
	c = document.createElement("canvas");
	ctx = c.getContext("2d");
	ctx.font = fontSize + ' ' + fontName;
	return ctx.measureText(text).width;
}

function AddComma(data_value) {
	return Number(data_value).toLocaleString('en');
}

var margin_si = {
		top: 0,
		right: 80,
		bottom: 80,
		left: 60
	},
	svg_si = d3.select("#chart_si"),
	width = svg_si.attr("width") - margin_si.left - margin_si.right,
	height = svg_si.attr("height") - margin_si.top - margin_si.bottom,
	g = svg_si.append("g").attr("transform", "translate(" + margin_si.left + "," + margin_si.top + ")"),
	textWidthHolder = 0;

//그래프 영역 지정
svg_si.append("defs").append("clipPath")
	.attr("id", "clip")
	.append("rect")
	.attr("width", width)
	.attr("height", +svg_si.attr("height"))
	.attr("transform", "translate(" + margin_si.left + "," + margin_si.top + ")");

//카테고리 첫번째 항목에 bar 항목 추가
var Categories = [];

Categories.pro;

array_si[0].Categories.forEach(function(b) {
	b.Type = "bar";
	Categories.push(b)
});

//data 불러오기
var array_si;

//zoom 비율 설정
if (array_si.length < 7) {
	zoom_rat = 1;
} else {
	zoom_rat = 1 * array_si.length / 7
}

//x축 그룹(샘플명)값 간격 설정
var x0 = d3.scaleBand()
	.rangeRound([margin_si.left, width])
	.paddingInner(0.1)
	.paddingOuter(0.1);

//x축 그룹 내 요소간(Q20/30) 간격 설정
var x1 = d3.scaleBand()
	.paddingOuter(0.25)
	.paddingInner(0.15);

//y축 값 간격 설정
var y = d3.scaleLinear()
	.domain([0, d3.max(array_si, function(d) {
		return d3.max(d.Categories, function(b) {
			return Number(b.Value);
		})
	})])
	.range([height, margin_si.bottom]);

//그래프 색 지정
var color = d3.scaleOrdinal()
	.range(["#5F79A9", "#70B9C8"]);

//그래프 상단 텍스트 색 지정
var color_text = d3.scaleOrdinal()
	.range(["#5F79A9", "#02a5b3"]);

//축 설정
var xAxis = d3.axisBottom(x0);
var yAxis = d3.axisLeft(y).ticks(7);

x0.domain(array_si.map(function(d) {
	return d.Sample;
}));
x1.domain(Categories.filter(function(d) {
	return d.Type == "bar"
}).map(function(d) {
	return d.Name
})).rangeRound([0, x0.bandwidth()]);

g.append("g")
	.style('clip-path', 'url(#clip)')
	.append("g")
	.attr("class", "x_axis")
	.attr("transform", "translate(" + margin_si.left + "," + height + ")")
	.call(xAxis).selectAll(".tick text")
	.call(wrap, x0.bandwidth());

g.append("g")
	.attr("transform", "translate(" + margin_si.left + ",0)")
	.attr("class", "y_axis")
	.call(yAxis)
	.append("text")
	.attr("transform", "rotate(0)")
	.attr("x", margin_si.left - 30)
	.attr("y", margin_si.top + 50)
	.attr("dy", ".71em")
	.style("text-anchor", "end")
	.attr("fill", "#000")
	.text("Number of Variants").call(wrap, margin_si.left - 10);

//축 텍스트 폰트 크기 설정
g.select(".x_axis")
	.selectAll("text")
	.style("font-size", "12px");
g.select(".y_axis")
	.selectAll("text")
	.style("font-size", "12px");

//bar 그래프
var state_si = g.append("g")
	.style('clip-path', 'url(#clip)')
	.selectAll(".state_si")
	.data(array_si)
	.enter().append("g")
	.attr("class", "state_si")
	.attr("transform", function(d) {
		return "translate(" + margin_si.left + x0(d.Sample) + ",0)";
	});

state_si.selectAll("rect")
	.data(function(d) {
		return d.Categories;
	})
	.enter().append("rect")
	.attr("class", "bar")
	.attr("width", x1.bandwidth())
	.attr("x", function(d) {
		return x1(d.Name);
	})
	.attr("y", function(d) {
		return y(d.Value);
	})
	.style("fill", function(d) {
		return color(d.Name);
	})
	.transition().attrTween("height", function(d) {
		var i = d3.interpolate(0, height - y(d.Value));
		return function(t) {
			return i(t);
		}
	});

//그래프 상단 텍스트
state_si.selectAll("text")
	.data(function(d) {
		return d.Categories;
	})
	.enter().append("text")
	.attr("x", function(d) {
		return x1(d.Name);
	})
	.attr("y", function(d) {
		return y(d.Value) - 2;
	})
	.style('fill', function(d) {
		return color_text(d.Name);
	})
	.style('font-size', '1.0em')
	.text(function(d) {
		return AddComma(parseFloat(d.Value).toFixed(0));
	});

//zoom in/out
var zoom_si = d3.zoom()
	.scaleExtent([1, Infinity])
	.translateExtent([
		[0, 0],
		[width, height]
	])
	.extent([
		[0, 0],
		[width, height]
	])
	.on("zoom", function() {
		return zoomed_si();
	});

svg_si.call(zoom_si)
	.call(zoom_si.transform, d3.zoomIdentity.translate(0, 50).scale(zoom_rat))

function zoomed_si() {
	x0.range([0, width].map(function(d) {
		return d3.event.transform.applyX(d);
	}));
	x1.rangeRound([0, x0.bandwidth()]);

	svg_si.selectAll(".state_si").attr("transform", function(d) {
		return "translate(" + x0(d.Sample) + ",0)";
	});
	state_si.selectAll(".bar").attr("x", function(d) {
		return margin_si.left + x1(d.Name);
	}).attr("width", x1.bandwidth());
	state_si.selectAll("text").attr("x", function(d) {
		return margin_si.left + x1(d.Name);
	}).attr("width", x1.bandwidth());
	svg_si.select(".x_axis").call(xAxis);
}

//reset
d3.select("#reset_si")
	.on("click", resetted);

function resetted() {
	svg_si.transition()
		.call(zoom_si.transform, d3.zoomIdentity.translate(0, 50).scale(zoom_rat));
}

//legend
var LegendHolder = g.append("g").attr("class", "legendHolder");
var legend = LegendHolder.selectAll(".legend")
	.data(Categories.map(function(d) {
		return {
			"Name": d.Name,
			"Type": d.Type
		}
	}))
	.enter().append("g")
	.attr("class", "legend")
	.attr("transform", function(d, i) {
		return "translate(0," + (height + margin_si.bottom / 2) + ")";
	})
	.each(function(d, i) {
		//legend 사각형
		d3.select(this).append("rect")
			.attr("width", function() {
				return 18
			})
			.attr("x", function(b) {

				left = (i + 1) * 15 + i * 18 + i * 5 + textWidthHolder;
				return left;
			})
			.attr("y", function(b) {
				return b.Type == 'bar' ? 0 : 7
			})
			.attr("height", function(b) {
				return b.Type == 'bar' ? 18 : 5
			})
			.style("fill", function(b) {
				return b.Type == 'bar' ? color(d.Name) : LineColor(d.Name)
			});

		//legend 텍스트
		d3.select(this).append("text")
			.attr("x", function(b) {
				left = (i + 1) * 15 + (i + 1) * 18 + (i + 1) * 5 + textWidthHolder;
				return left;
			})
			.attr("y", 9)
			.attr("dy", ".35em")
			.style("text-anchor", "start")
			.text(d.Name);
		textWidthHolder += getTextWidth(d.Name, "10px", "calibri");
	});

//legend 위치
d3.select(".legendHolder").attr("transform", function(d) {
	thisWidth = d3.select(this).node().getBBox().width;
	return "translate(" + ((width) / 2 - thisWidth / 2) + ",0)";
});

//텍스트 줄바꿈
function wrap(text, width) {
	text.each(function() {
		var text = d3.select(this),
			words = text.text().split(/\s+/).reverse(),
			word,
			line = [],
			lineNumber = 0,
			lineHeight = 1.1, //ems
			y = text.attr("y"),
			dy = parseFloat(text.attr("dy")),
			tspan = text.text(null).append("tspan").attr("x", 0).attr("y", y).attr("dy", dy + "em");
		while (word = words.pop()) {
			line.push(word);
			tspan.text(line.join(" "));
			if (tspan.node().getComputedTextLength() > width) {
				line.pop();
				tspan.text(line.join(" "));
				line = [word];
				tspan = text.append("tspan").attr("x", 0).attr("y", y).attr("dy", (++lineNumber * lineHeight + dy) + "em").text(word);
			}
		}
	});
}
